<?php


if (!function_exists('json_decode')) {
	include(dirname(__FILE__)  . '/json_compat_decode.php');
}

if (!function_exists('json_encode')) {
	include(dirname(__FILE__)  . '/json_compat_encode.php');
}


class faq_data {

	static public $errors = array();
	static public $connection;

	static public function initDb () {
		$connection = self::connect();

		if(!$connection) {
			die('Unable to connect to database. Mysql said: ' . self::lastError());
		}

		self::$connection = $connection;

		self::query("SET NAMES 'utf8'");
	}

	static public function lastError() {
		$sizeOf = sizeof(self::$errors);
		if($sizeOf == 0) {
			return '';
		}
		$sizeOf = $sizeOf - 1;
		return self::$errors[$sizeOf];
	}

	static public function connect () {
		$link = @mysql_connect(FAQ_DB_HOST, FAQ_DB_USER, FAQ_DB_PASS);
		if (!$link) {
			self::logError();
			$vars = array('message'=> "Please ensure your database username, password, host name and database name are correct. The database must already exist.");
			$html = file_get_contents(BASE_DIR . '/faq_admin/display/database_error.phtml');
			$parse = new faq_display_parse($html, $vars);
			echo $parse->parse();
			die();
		}

		$dbSelected = @mysql_select_db(FAQ_DB_NAME, $link);
		if (!$dbSelected) {
		    self::logError();
			$vars = array('message'=> "The database '" . htmlentities(FAQ_DB_NAME) . "' doesn't appear to exist. Please create this database and try again.");
			$html = file_get_contents(BASE_DIR . '/faq_admin/display/database_error.phtml');
			$parse = new faq_display_parse($html, $vars);
			echo $parse->parse();
			die();
		}

		return $link;
	}

	static public function query ($query) {
		$query = str_replace(':categories_table', '`' . self::tableName('categories') . '`', $query);
		$query = str_replace(':questions_table', '`' . self::tableName('questions') . '`', $query);
		$query = str_replace(':users_table', '`' . self::tableName('users') . '`', $query);
		$query = str_replace(':settings_table', '`' . self::tableName('settings') . '`', $query);

		$queryResult = mysql_query($query);
		if(!$queryResult) {
			self::logError();
			return false;
		}
		return $queryResult;
	}

	static public function row($query) {
		$result = false;

		if(is_string($query)) {
			$result = self::query($query);
		} else if (is_resource($query)) {
			$result = $query;
		}

		if(!$result) {
			return false;
		}

		return mysql_fetch_assoc($result);
	}


	static public function updateQuestion ($questionId, $question, $answer, $categoryId) {
		if((int)$categoryId <= 0) {
			return false;
		}

		$sql = 'UPDATE `' . self::tableName('questions') . '` SET ';
		$sql .= '`question` = "' . self::escape($question) . '",';
		$sql .= '`answer` = "' . self::escape($answer) . '",';
		$sql .= '`categoryid` = "' . self::escape($categoryId) . '"';
		$sql .= ' where `questionid` = ' . (int)$questionId;

		return self::query($sql);
	}

	static public function updateUser ($userId, $username, $password) {
		if((int)$userId <= 0) {
			return false;
		}

		$sql = 'UPDATE :users_table SET ';
		$sql .= '`username` = "' . self::escape($username) . '"';
		if($password !== false) {
			$sql .= ',`password` = "' . faq_user::hashPass($password) . '"';
		}
		$sql .= ' where `id` = ' . (int)$userId;

		return self::query($sql);
	}

	static public function deleteQuestion ($questionId) {
		if((int)$questionId <= 0) {
			return false;
		}

		$sql = 'DELETE FROM `' . self::tableName('questions') . '`';
		$sql .= ' where `questionid` = ' . (int)$questionId;

		return self::query($sql);
	}


	static public function deleteCategory ($categoryId) {
		if((int)$categoryId <= 0) {
			return false;
		}

		$current_category = self::row('SELECT * FROM :categories_table WHERE categoryid = ' . (int) $categoryId );


		$children  = self::query('SELECT * FROM :categories_table WHERE LEFT(`hierarchy`, ' . (strlen($current_category['hierarchy'])+1) . ') = "' . self::escape($current_category['hierarchy']) . '." AND categoryid != ' . (int)$current_category['categoryid'] . ' ORDER BY `hierarchy` ASC');

		$removeIds = array($categoryId);

		while($child = self::row($children)) {
			$removeIds[] = $child['categoryid'];
		}

		$sql = 'DELETE FROM :categories_table';
		$sql .= ' where `categoryid` IN (' . implode(',', $removeIds) . ')';

		$return = self::query($sql);

		$sql = 'DELETE FROM :questions_table';
		$sql .= ' where `categoryid` IN (' . implode(',', $removeIds) . ')';

		$return_questions = self::query($sql);

		if($return_questions && $return) {
			return true;
		}

		return false;
	}

	static public function updateQuestionOrder ($questionId, $orderNumber) {
		if((int)$questionId <= 0) {
			return false;
		}

		$sql = 'UPDATE `' . self::tableName('questions') . '`';
		$sql .= ' SET `sortorder` ="' . (int)$orderNumber . '"';
		$sql .= ' where `questionid` = ' . (int)$questionId;

		return self::query($sql);
	}

	static public function updateCategory ($categoryId, $name, $parent = 0) {
		if((int)$categoryId <= 0) {
			return false;
		}

		$parent = (int)$parent;
		if($parent < 0){
			$parent = 0;
		}

		$current_category = self::row('SELECT c.name as cat_name, c.categoryid as cat_id, c.hierarchy as hierarchy, p.name as parent_name, p.categoryid as parent_id FROM :categories_table as c LEFT JOIN `faq_categories` as p ON LEFT(c.hierarchy, LENGTH(c.hierarchy)-3) = p.hierarchy WHERE c.categoryid = ' . (int) $categoryId );

		if(empty($current_category['parent_id'])) {
			$current_category['parent_id'] = 0;
		}

		if($current_category['parent_id'] != $parent) {
			// parent category has changed... oh dear

			// firstly we need to verify that the new parent category isn't a child of our current category (infinite loop)
			$check_row = self::row('SELECT * FROM :categories_table WHERE hierarchy LEFT(`hierarchy`, ' . strlen($current_category['hierarchy']) . ') = "' . self::escape($current_category['hierarchy']) . '" AND categoryid = ' . (int)$parent);

			if(!empty($check_row)) {
				self::error('Invalid parent category. You can not select a sub-category of this category as it\'s parent category.');
				return false;
			}

			// get the hierarchy code for it's new position
			$hierarchy = self::getHierarchyCode($parent);

			$sql = 'UPDATE :categories_table SET ';
			$sql .= '`name` ="' . self::escape($name) . '", ';
			$sql .= '`hierarchy` ="' . self::escape($hierarchy) . '"';
			$sql .= ' where `categoryid` = ' . (int)$categoryId;

			if(!self::query($sql)) {
				return false;
			}

			// update all the children
			$children  = self::query('SELECT * FROM :categories_table WHERE LEFT(`hierarchy`, ' . (strlen($current_category['hierarchy'])+1) . ') = "' . self::escape($current_category['hierarchy']) . '." AND categoryid != ' . (int)$current_category['cat_id'] . ' ORDER BY `hierarchy` ASC');

			while($child = self::row($children)) {
				$child['hierarchy'] = substr($child['hierarchy'], strlen($current_category['hierarchy']));
				$child['hierarchy'] = $hierarchy . $child['hierarchy'];

				$sql = 'UPDATE :categories_table';
				$sql .= ' SET `hierarchy` ="' . self::escape($child['hierarchy']) . '"';
				$sql .= ' where `categoryid` = ' . (int)$child['categoryid'];

				self::query($sql);

			}

			return true;

		} else {
			// just updating the name

			$sql = 'UPDATE :categories_table';
			$sql .= ' SET `name` ="' . self::escape($name) . '"';
			$sql .= ' where `categoryid` = ' . (int)$categoryId;

			return self::query($sql);
		}
	}

	static public function updateCategoryOrder ($categoryId,$heirarchy) {
		$sql = 'UPDATE :categories_table';
		$sql .= ' SET `hierarchy` ="' . self::escape($heirarchy) . '"';
		$sql .= ' where `categoryid` = ' . (int)$categoryId;

		return self::query($sql);
	}

	static public function getHierarchyCode ($parent, $hierarchy = '00') {
		$h = new faq_hierarchy();

		if($parent == 0) {
			// no parent category, add it to the bottom of the root categories
			$last_root_cat = self::row('SELECT * FROM :categories_table WHERE LENGTH(`hierarchy`)=2 ORDER BY `hierarchy` DESC LIMIT 1');
			if(!empty($last_root_cat)) {
				// no categories by the looks of it!
				$number = (int)$last_root_cat['hierarchy'];
				++$number;
				$hierarchy = $h->numberToString($number);
			}
		} else {
			$parent_cat = self::row('SELECT * FROM :categories_table WHERE categoryid = ' . $parent);

			if(empty($parent_cat)) {
				self::error('The selected parent category doesn\'t exist.');
				return false;
			}

			// problem with this logic what if the last one is a sub-sub-cat ?? need to restrict on the length
			// need to test if fix worked
			$last_sibling  = self::row('SELECT * FROM :categories_table WHERE LEFT(`hierarchy`, ' . strlen($parent_cat['hierarchy']) . ') = "' . self::escape($parent_cat['hierarchy']) . '" AND categoryid != ' . (int)$parent_cat['categoryid'] . ' AND LENGTH(`hierarchy`) = ' . (strlen($parent_cat['hierarchy'])+3) . ' ORDER BY `hierarchy` DESC LIMIT 1');

			if(empty($last_sibling)) {
				// first child
				$hierarchy = $parent_cat['hierarchy'] . ".00";
			} else {

				$levels = explode('.', $last_sibling['hierarchy']);

				$currentLevel = array_pop($levels);
				$currentLevel = (int)$currentLevel;
				++$currentLevel;
				$levels[] = $h->numberToString($currentLevel);

				$hierarchy = implode('.', $levels);
			}
		}

		return $hierarchy;
	}

	static public function addCategory ($name, $parent = 0) {
		$parent = (int)$parent;

		if($parent < 0) {
			$parent = 0;
		}

		$hierarchy = self::getHierarchyCode($parent);

		if(empty($hierarchy)) {
			return false;
		}

		$sql = 'INSERT INTO :categories_table (`name`, `hierarchy`) ';
		$sql .= ' VALUES (';
		$sql .= '"' . self::escape($name) . '",';
		$sql .= '"' . self::escape($hierarchy) . '"';
		$sql .= ')';

		return self::query($sql);
	}

	static public function addQuestion ($question, $answer, $categoryId) {

		$maxSort = self::single('SELECT MAX(`sortorder`) + 1 FROM `' . self::tableName('questions') . '` where categoryid =  ' . (int)$categoryId);

		$sql = 'INSERT INTO `' . self::tableName('questions') . '` (`question`, `answer`, `categoryid`, `sortorder`) ';
		$sql .= ' VALUES (';
		$sql .= '"' . self::escape($question) . '",';
		$sql .= '"' . self::escape($answer) . '",';
		$sql .= (int)$categoryId . ',';
		$sql .= (int)$maxSort;

		$sql .= ')';

		return self::query($sql);
	}

	static public function addUser ($username, $password) {

		$sql = 'INSERT INTO `' . self::tableName('users') . '` (`username`, `password`) ';
		$sql .= ' VALUES (';
		$sql .= '"' . self::escape($username) . '",';
		$sql .= '"' . faq_user::hashPass($password) . '"';
		$sql .= ')';

		return self::query($sql);
	}

	static public function single ($query) {
		return mysql_result(self::query($query), 0, 0);
	}

	static public function escape ($string) {
		if (get_magic_quotes_gpc()) {
			$string = stripslashes($string);
		}

		$string = str_replace("<script", "&lt;script", $string );
		$string = str_replace("</script", "&lt;/script", $string );

		return mysql_real_escape_string($string);
	}

	static public function questionById($id) {
		if((int)$id <= 0) {
			return false;
		}
		return self::row('select * from `' . self::tableName('questions') . '` where questionid = ' . (int)$id);
	}

	static public function categoryById($id) {
		if((int)$id <= 0) {
			return false;
		}
		return self::row('select * from :categories_table where questionid = ' . (int)$id);
	}

	static public function tableName ($table) {
		return FAQ_DB_PREFIX . $table;
	}

	static private function error($error) {
		self::$errors[] = $error;
	}

	static private function logError() {
		self::$errors[] = mysql_error();
	}


}

faq_data::initDb();
