<?php
/**
 * @package FAQ_Manager
 * @version 1.0
 */
/*
Plugin Name: FAQ Manager
Plugin URI: http://www.jordie.org/faq-manager.php?w
Description: The plug-in for FAQ Manager
Author: Jordie Bodlay
Version: 1.0
Author URI: http://www.jordie.org/
*/

add_filter ( 'the_content', 'show_faqmanager');


add_action('admin_init', 'faqmanager_admin_init');

function faqmanager_gettypes() {
	return array('accordion', 'multipage', 'singlepage');
}

function show_faqmanager($content) {
	if(get_post_type() == "page") {
		$options = get_option('faqmanager_options');

		if(!in_array($options['faqmanager_type'], faqmanager_gettypes())) {
			$options['faqmanager_type'] = 'accordion';
		}

		if (get_the_ID() == $options['faq_page_id']) {
			ob_start();
			include($options['faqmanager_path'] . "faq_".$options['faqmanager_type'] .".php");
			$content = ob_get_contents();
			ob_end_clean();
		}
	}
	return $content;
}

function faqmanager_admin_init(){
	register_setting( 'faqmanager_options', 'faqmanager_options', 'faqmanager_options_validate' );

	add_settings_section('faqmanager_plugin_main', '', 'faqmanager_text', 'faqmanager_plugin');
	add_settings_field('faqmanager_path', 'FAQ Manager Path', 'faqmanager_path_field', 'faqmanager_plugin', 'faqmanager_plugin_main');
	add_settings_field('faq_page_id', 'Display FAQ on Page', 'faqmanager_page_id_field', 'faqmanager_plugin', 'faqmanager_plugin_main');
	add_settings_field('faqmanager_type_field', 'FAQ Display Mode', 'faqmanager_type_field', 'faqmanager_plugin', 'faqmanager_plugin_main');
}

add_action('admin_menu', 'faqmanager_menu');

function faqmanager_menu() {
	add_options_page('FAQ Manager Options', 'FAQ Manager', 'manage_options', 'faqmanager', 'faqmanager_options_page');
}

function faqmanager_text() {
//	echo '<p>Main description of this section here.</p>';
}

function faqmanager_path_field() {
	$options = get_option('faqmanager_options');
	echo "<input id='faqmanager_path' name='faqmanager_options[faqmanager_path]' size='40' type='text' value='{$options['faqmanager_path']}' /> <input type='button' id='faqmanager_findme' value='Find it for me' /> <span class='description'>This should be the server path to the directory where the file faq_init.php is.</span>";
}

function faqmanager_page_id_field() {
	$options = get_option('faqmanager_options');

	$pages = get_pages( array( 'sort_column' => 'menu_order' ) );
	echo '<select id="faqmanager_pages" name="faqmanager_options[faq_page_id]">';
	foreach($pages as $page) {
		if($page->ID == $options['faq_page_id']) {
			$selected = "selected='selected'";
		} else {
			$selected = "";
		}
		echo '<option value="'.$page->ID.'" '.$selected.'>'.$page->post_title.'</option>' . "\n";
	}
	echo "</select>";
}


function faqmanager_type_field() {
	$options = get_option('faqmanager_options');
	?>
	<select id="faqmanager_type" name="faqmanager_options[faqmanager_type]">
		<option value="accordion" <?php if($options['faqmanager_type'] == 'accordion') { echo "selected='selected'"; } ?>>Accordion</option>
		<option value="multipage" <?php if($options['faqmanager_type'] == 'multipage') { echo "selected='selected'"; } ?>>Multi-Page</option>
		<option value="singlepage" <?php if($options['faqmanager_type'] == 'singlepage') { echo "selected='selected'"; } ?>>Single-Page</option>
	</select>
	<?php
}


function faqmanager_options_validate($input) {
	$newinput['faqmanager_path'] = '';

	$input['faqmanager_path'] = trim($input['faqmanager_path']);

	if(is_file($input['faqmanager_path'])) {
		$path_parts = pathinfo($input['faqmanager_path']);
		$input['faqmanager_path'] = $path_parts['dirname'] . '/';
	}

	if(substr($input['faqmanager_path'], -1) != '/') {
		$input['faqmanager_path'] .= '/';
	}

	if(is_file($input['faqmanager_path'] . 'faq_init.php')) {
		$newinput['faqmanager_path'] = $input['faqmanager_path'];
	} else {
		add_settings_error('faqmanager_path', "faqmanager_path_error", "Error: The path you entered was not valid.", 'error');
		add_action('admin_notices', 'faqmanager_bad_path');
	}

	$newinput['faq_page_id'] = (int)trim($input['faq_page_id']);
	$newinput['faqmanager_type'] = trim($input['faqmanager_type']);

	if(!in_array($newinput['faqmanager_type'], faqmanager_gettypes())) {
		$newinput['faqmanager_type'] = 'accordion';
	}

	return $newinput;
}

function faqmanager_options_page () {
	?>
	<div class="wrap">
	<div id="icon-options-general" class="icon32">
	<br>
	</div>
	<h2>FAQ Manager Settings</h2>
	<form action="options.php" method="post">
	<?php settings_fields('faqmanager_options'); ?>
	<?php do_settings_sections('faqmanager_plugin'); ?>
	<p class="submit">
	<input id="submit" class="button-primary" type="submit" value="<?php esc_attr_e('Save Changes'); ?>" name="submit">
	</p>

	</form></div>

	<?php
}

add_action('admin_head', 'faqmanager_javascript');

function faqmanager_javascript() {
?>
<script type="text/javascript" >
jQuery(document).ready(function($) {

	var data = {
		action: 'faqmanager_findme'
	};

	$('#faqmanager_findme').click(function () {
		$('#faqmanager_findme').after('<img src="images/loading.gif" id="faqmanager_loading" />');
		jQuery.post(ajaxurl, data, function(json) {
			if(json.success) {
				$('#faqmanager_path').val(json.path);
			} else {
				var msg = "You will have to manually enter the path to FAQ Manager. \n\nYou can locate this path by clicking 'View FAQ' in the FAQ Manager control panel.";
				if(json.error == 'timeout') {
					alert("Sorry, I was unable to locate FAQ Manager within the allowed timeframe. " + msg);
				} else if (json.error == 'notfound'){
					alert("Sorry, I was unable to locate FAQ Manager on your server." + msg);
				}
			}
			$('#faqmanager_loading').remove();
		}, 'json');
	});
});

</script>
<?php
}


add_action('wp_ajax_faqmanager_findme', 'faqmanager_findme_callback');

function faqmanager_findme_callback() {
	$GLOBALS['faqmanager_findme_start_time'] = microtime(true);
	$GLOBALS['faqmanager_findme_timeout_seconds'] = 10;

	$dir = str_replace("\\", "/", str_replace('wp-admin/admin-ajax.php', '', $_SERVER['REQUEST_URI']));
	$path = str_replace("\\", "/", ABSPATH);
	$minus = 0 - strlen($dir);
	$end_path = substr($path, $minus);
	if($end_path == $dir){
		$start_path = substr($path, 0, (strlen($path) - strlen($end_path)));
		if(substr($start_path, -1) != '/') {
			$start_path .= '/';
		}
	}

	$ignore = array(
		'images',
		'wp-admin',
		'admin',
		'themes',
		'css',
		'js'
		);

	faqmanager_findme($start_path, $ignore);
	//	$ignore[] = basename($start_path);
	//	faqmanager_findme(dirname($start_path) . '/', $ignore);

	echo json_encode(array('success' => false, 'error' => 'notfound'));
	die();
}

function faqmanager_checktime() {
	if((microtime(true) - $GLOBALS['faqmanager_findme_start_time']) > $GLOBALS['faqmanager_findme_timeout_seconds']) {
		echo json_encode(array('success' => false, 'error' => 'timeout'));
		die();
	}
}

function faqmanager_found($dir) {
		echo json_encode(array('success' => true, 'path' => $dir));
		die();
}

function faqmanager_findme ($dir, $ignore=array()) {

	faqmanager_checktime();

	$mydir = opendir($dir);
	while(false !== ($file = readdir($mydir))) {
		if($file == "faq_init.php") {
			faqmanager_found($dir);
		} else if($file != "." && $file != "..") {
			if(is_dir($dir.$file) && !in_array($file, $ignore)) {
				faqmanager_findme($dir.$file.'/', $ignore);
			}
		}
	}
	closedir($mydir);
}